<?php

namespace Drupal\email_validate\Form;

use Drupal\Core\DrupalKernel;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\TypedData\TypedDataTrait;

/**
 * Configure Email extended validation settings for this site.
 */
class EmailValidateForm extends ConfigFormBase {

  use TypedDataTrait;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'email_validate_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['email_validate.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $configs = $this->config('email_validate.settings')
      ->getRawData();
    $constraints = $this->getConstraintsAll();

    foreach ($constraints as $constraintKey) {
      $constraintClass = 'Drupal\email_validate\Plugin\Validation\Constraint\\' . $constraintKey;
      $constraint = new $constraintClass();
      $constraintSettings = $configs[$constraintKey] ?? [];
      $is_enable = !empty($constraintSettings['enable']);

      $form[$constraintKey] = [
        '#title' => $constraint->title,
        '#type' => 'details',
        '#open' => $is_enable,
        '#tree' => TRUE,
      ];

      $form[$constraintKey]['enable'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Enable "@name".', ['@name' => $constraint->title]),
        '#default_value' => $is_enable,
        '#description' => $constraint->description,
      ];

      // Add constraint settings fields.
      $form[$constraintKey] += $constraint->getSettingsForm($constraintSettings);
    }

    // Collapsible details.
    $form['#attached']['library'][] = 'core/drupal.collapse';
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $constraints = $this->getConstraintsAll();
    $values = $form_state->getValues();

    foreach ($constraints as $constraint) {
      $this->config('email_validate.settings')
        ->set($constraint, $values[$constraint])
        ->save();
    }

    parent::submitForm($form, $form_state);

    // Reset field base info cache.
    $this->resetCache();
  }

  /**
   * Update field base info for applying the changes.
   *
   * See drupal_flush_all_caches().
   */
  private function resetCache() {
    $kernel = NULL;
    if (!$kernel instanceof DrupalKernel) {
      $kernel = \Drupal::service('kernel');
      $kernel->invalidateContainer();
      $kernel->rebuildContainer();
    }

    // Clear all plugin caches.
    \Drupal::service('plugin.cache_clearer')->clearCachedDefinitions();
  }

  /**
   * Return all available Constraints for email field.
   *
   * @return array|int[]|string[]
   *   Array of constraints plugins.
   */
  private function getConstraintsAll() {
    static $cache = [];

    if (!$cache) {
      /**
       * @var \Drupal\Core\File\FileSystemInterface $file_system
       */
      $file_system = \Drupal::service('file_system');
      $module_handler = \Drupal::service('module_handler');
      $module_path = $module_handler->getModule('email_validate')->getPath();
      $module_driver_path = DRUPAL_ROOT . '/' . $module_path . '/src/Plugin/Validation/Constraint';
      if (is_dir($module_driver_path)) {
        $mask = '/.*Constraint.php$/';
        $constraints = $file_system->scanDirectory(
          $module_driver_path, $mask, [
            'key' => 'name',
            'recurse' => FALSE,
          ]
        );
      }
      $cache = array_keys($constraints);
    }

    return $cache;
  }

}
